import os
import shutil
import subprocess

def download_nuget(version: str = "", update: bool = False) -> None:
    nuget_path = os.path.join(os.path.dirname(__file__), 'aacommpyDownloader-main', 'nuget.exe')
    installed = False
    for dirname in os.listdir(os.path.dirname(nuget_path)):
        if dirname.startswith('Agito.AAComm.') and os.path.isdir(os.path.join(os.path.dirname(nuget_path), dirname)):
            installed = True
            old_version = dirname.split('.')[2:]
            old_version = '.'.join(old_version)
            break
    if update and installed:
        shutil.rmtree(os.path.join(os.path.dirname(nuget_path), f'Agito.AAComm.{old_version}'))
    nuget_cmd = [nuget_path, 'install', 'Agito.AAComm', '-OutputDirectory', os.path.join(os.path.dirname(nuget_path)), '-Source', 'https://api.nuget.org/v3/index.json']
    if version != "":
        nuget_cmd.extend(['-Version', version])
    subprocess.run(nuget_cmd, check=True)
    for dirname in os.listdir(os.path.dirname(nuget_path)):
        if dirname.startswith('Agito.AAComm.') and os.path.isdir(os.path.join(os.path.dirname(nuget_path), dirname)):
            new_version = dirname.split('.')[2:]
            new_version = '.'.join(new_version)
            source_dir = os.path.join(os.path.dirname(nuget_path), f'Agito.AAComm.{new_version}/build/AACommServer')
            dest_dir = os.path.dirname(__file__)
            shutil.copy2(os.path.join(source_dir, 'AACommServer.exe'), dest_dir)
            shutil.copy2(os.path.join(source_dir, 'AACommServerAPI.dll'), dest_dir)
            source_dir2 = os.path.join(os.path.dirname(nuget_path), f'Agito.AAComm.{new_version}/lib/net5.0')
            shutil.copy2(os.path.join(source_dir2, 'AAComm.dll'), dest_dir)
            break
    return None
def nuget_version() -> str:
    nuget_path = os.path.join(os.path.dirname(__file__), 'aacommpyDownloader-main', 'nuget.exe')
    nuget_cmd = [nuget_path, 'list', 'Agito.AAComm', '-AllVersions']
    result = subprocess.run(nuget_cmd, check=True, stdout=subprocess.PIPE, text=True)
    versions = []
    for line in result.stdout.splitlines():
        if 'Agito.AAComm ' in line:
            version = line.split(' ')[-1]
            versions.append(version)
    if len(versions) > 0:
        latest_version = versions[-1]
        print(f'The installed versions of Agito.AAComm are: {", ".join(versions)}.')
        return latest_version
    else:
        raise RuntimeError('Agito.AAComm package is not installed.')
def update_nuget() -> None:
    download_nuget(update=True)
    return None
def dotnetfw(version: str = "net5.0") -> None:
    latest_version = nuget_version()
    source_dir = os.path.join(os.path.dirname(__file__), 'aacommpyDownloader-main', f'Agito.AAComm.{latest_version}')
    dest_dir = os.path.dirname(__file__)    
    if version == "net5.0":
        source_dir = os.path.join(source_dir, 'lib', 'net5.0')
    elif version == "net40":
        source_dir = os.path.join(source_dir, 'lib', 'net40')
    elif version == "net46":
        source_dir = os.path.join(source_dir, 'lib', 'net46')
    elif version == "net48":
        source_dir = os.path.join(source_dir, 'lib', 'net48')
    elif version == "netcoreapp3.1":
        source_dir = os.path.join(source_dir, 'lib', 'netcoreapp3.1')
    else:
        raise ValueError("Invalid .NET framework version specified.")    
    dll_path = os.path.join(source_dir, 'AAComm.dll')
    if not os.path.isfile(dll_path):
        raise FileNotFoundError(f"Could not find AAComm.dll in {source_dir}.")
    shutil.copy2(dll_path, dest_dir)
    return None


import clr
import os
path = os.path.join(os.path.dirname(__file__), 'aacommpyDownloader-main', 'Agito.AAComm.5.5.0','lib','net48','AAComm.dll')
# clr.AddReference(path)
from AAComm import CommAPI, ConnectResult, ConnectionData


class CommAPIClient:
    def __init__(self):
        clr.AddReference(path)
        self.api = CommAPI()
        status = self.api.StartAACommServer()
        if status != "":
            raise Exception("Failed to start AACommServer: " + status)
    
    def connect(self, ip_address, port):
        cData = ConnectionData()
        cData.ControllerType = Shared.ProductTypes.AGM800_ID
        cData.CommChannelType = Shared.ChannelType.Ethernet
        cData.ET_IP_1, cData.ET_IP_2, cData.ET_IP_3, cData.ET_IP_4 = ip_address.split('.')
        cData.ET_Port = port
        res = self.api.Connect(cData)
        return res



import argparse
import os
import requests
from zipfile import ZipFile
import subprocess
import shutil
from .nugetmanagement import download_nuget, nuget_version, update_nuget, dotnetfw

entry_points = {
    'console_scripts': [
        'nuget_download = nugetmanagement:download_nuget',
        'nuget_version = nugetmanagement:nuget_version',
        'update_nuget = nugetmanagement:update_nuget'
    ]
}
def install_template() -> None:
    msg = 'Download Python package for AAComm to this directory (y/n)? '
    user_response = input(msg)
    if user_response != 'y':
        return None
    url = 'https://github.com/jamesbond90/aacommpyDownloader/archive/refs/heads/main.zip'
    r = requests.get(url)
    with open(os.path.join(os.path.dirname(__file__), 'main.zip'), 'wb') as f:
        f.write(r.content)

    with ZipFile(os.path.join(os.path.dirname(__file__), 'main.zip'), 'r') as repo_zip:
        repo_zip.extractall(os.path.dirname(__file__))

    os.remove(os.path.join(os.path.dirname(__file__), 'main.zip'))
    return None

def main() -> None:
    parser = argparse.ArgumentParser(description='Download aacommpy package.')
    parser.add_argument('command', choices=['install', 'downloadnuget', 'version', 'update'], help='Choose a command to execute.')
    parser.add_argument('--version', help='Specify version to install/download.')
    args = parser.parse_args()

    if args.command == 'install':
        install_template()
    elif args.command == 'downloadnuget':
        if args.version:
            download_nuget(args.version)
        else:
            download_nuget()
    elif args.command == 'version':
        nuget_version()
    elif args.command == 'update':
        update_nuget()
    elif args.command == 'dotnetfw':
        dotnetfw()
    else:
        raise RuntimeError('Please supply a valid command for aacommpy - e.g. install.')

    return None

if __name__ == '__main__':
    main()
